==> dianara-v1.0.tar.gz: dianara-v1.0/src/composer.cpp <==
/*
 *   This file is part of Dianara
 *   Copyright 2012-2013  JanKusanagi <janjabber@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .
 */

#include "composer.h"


Composer::Composer(bool forPublisher)
{
    this->forPublisher = forPublisher;

    this->setAcceptRichText(true);
    this->setTabChangesFocus(true);

    // FIXME: do something about the links being styled with user's color
    //this->setStyleSheet("{ a:link { color: #FF0000;} }");

    QFont startConversationFont;
    startConversationFont.setPointSize(startConversationFont.pointSize() - 2);

    startConversationLabel = new QLabel(tr("Click here or press Control+N to post a note..."));
    startConversationLabel->setFont(startConversationFont);

    // A menu to insert some Unicode symbols
    symbolsMenu = new QMenu(tr("Symbols"));
    symbolsMenu->setIcon(QIcon::fromTheme("character-set"));
    symbolsMenu->addAction(QString::fromUtf8("\342\230\272")); // Smiling face
    symbolsMenu->addAction(QString::fromUtf8("\342\230\271")); // Sad face
    symbolsMenu->addAction(QString::fromUtf8("\342\231\245")); // Heart
    symbolsMenu->addAction(QString::fromUtf8("\342\231\253")); // Musical note
    symbolsMenu->addAction(QString::fromUtf8("\342\230\225")); // Coffee
    symbolsMenu->addAction(QString::fromUtf8("\342\234\224")); // Check mark
    symbolsMenu->addAction(QString::fromUtf8("\342\230\205")); // Black star
    symbolsMenu->addAction(QString::fromUtf8("\342\254\205")); // Arrow to the left
    symbolsMenu->addAction(QString::fromUtf8("\342\236\241")); // Arrow to the right
    connect(symbolsMenu, SIGNAL(triggered(QAction*)),
            this, SLOT(insertSymbol(QAction*)));


    toolsMenu = new QMenu(tr("Formatting"));
    toolsMenu->addAction(QIcon::fromTheme(""),
                         tr("Normal"),
                         this,
                         SLOT(makeNormal()));
    toolsMenu->addAction(QIcon::fromTheme("format-text-bold"),
                         tr("Bold"),
                         this,
                         SLOT(makeBold()),
                         QKeySequence::Bold);
    toolsMenu->addAction(QIcon::fromTheme("format-text-italic"),
                         tr("Italic"),
                         this,
                         SLOT(makeItalic()),
                         QKeySequence::Italic);
    toolsMenu->addAction(QIcon::fromTheme("format-text-underline"),
                         tr("Underline"),
                         this,
                         SLOT(makeUnderline()),
                         QKeySequence::Underline);
    toolsMenu->addAction(QIcon::fromTheme("format-text-strikethrough"),
                         tr("Strikethrough"),
                         this,
                         SLOT(makeStrikethrough()));

    toolsMenu->addSeparator();

    toolsMenu->addAction(QIcon::fromTheme("format-font-size-more"),
                         tr("Header"),
                         this,
                         SLOT(makeHeader()),
                         QKeySequence("Ctrl+H"));
    toolsMenu->addAction(QIcon::fromTheme("format-justify-fill"),
                         tr("Preformatted block"),
                         this,
                         SLOT(makePreformatted()));
    toolsMenu->addAction(QIcon::fromTheme("format-text-italic"),
                         tr("Quote block"),
                         this,
                         SLOT(makeQuote()));

    toolsMenu->addSeparator();

    toolsMenu->addAction(QIcon::fromTheme("insert-link"),
                         tr("Make a link"),
                         this,
                         SLOT(makeLink()),
                         QKeySequence("Ctrl+L"));
    toolsMenu->addAction(QIcon::fromTheme("insert-image"),
                         tr("Insert an image from a web site"),
                         this,
                         SLOT(insertImage()));
    toolsMenu->addAction(QIcon::fromTheme("insert-horizontal-rule"),
                         tr("Insert line"),
                         this,
                         SLOT(insertLine()));

    toolsMenu->addSeparator();

    toolsMenu->addMenu(symbolsMenu);

    toolsMenu->addSeparator();

    toolsMenu->addAction(QIcon::fromTheme("dialog-cancel"),
                         tr("Cancel Post"),
                         this,
                         SLOT(cancelPost()));

    toolsButton = new QPushButton(QIcon::fromTheme("format-list-ordered"),
                                  tr("&F", "F for formatting, translate accordingly"));
    toolsButton->setMenu(toolsMenu);
    toolsButton->setToolTip(tr("Text Formatting Options"));
    toolsButton->setHidden(true); // Hidden on startup (Publisher in miniMode)


    // Extra action for the context menu, paste as plaintext
    this->pastePlaintextAction = new QAction(QIcon::fromTheme("paste"),
                                             tr("Paste Text Without Formatting"),
                                             this);

    pastePlaintextAction->setShortcut(QKeySequence("Ctrl+Shift+V")); // This doesn't work like this
    connect(pastePlaintextAction, SIGNAL(triggered()),
            this, SLOT(pasteAsPlaintext()));

    // Add action to this object, in order for the shortcut to work
    // Otherwise, the action isn't available until the context menu is present
    this->addAction(pastePlaintextAction);

    mainLayout = new QGridLayout();


    if (this->forPublisher)  // Publisher mode
    {
        this->setToolTip(tr("Type a message here to post it"));

        mainLayout->addWidget(startConversationLabel, 0, 0, 1, 1, Qt::AlignLeft | Qt::AlignVCenter);
    }
    else                     // or Commenter mode
    {
        this->setToolTip(tr("Type a comment here"));
    }

    mainLayout->addWidget(toolsButton,            0, 1, 1, 1, Qt::AlignRight | Qt::AlignBottom);
    this->setLayout(mainLayout);

    qDebug() << "Composer box created";
}


Composer::~Composer()
{
    qDebug() << "Composer box destroyed";
}



void Composer::erase()
{
    this->clear();

    if (this->forPublisher)
    {
        startConversationLabel->show();

        toolsButton->hide();  /// FIXME: maybe move this line to focusOutEvent
    }
}


void Composer::insertLink(QString url, QString title)
{
    if (title.isEmpty())
    {
        title = url;
    }

    this->insertHtml("<a href=\"" + url + "\">"
                     "<span style=\"color:#0088CC\">"
                     + title + "</span></a> "); // Important, space after!
}


/*
 * Send a signal when getting focus
 *
 */
void Composer::focusInEvent(QFocusEvent *event)
{
    emit focusReceived(); // inform Publisher() or Commenter() that we have focus

    if (this->forPublisher)
    {
        startConversationLabel->hide(); // hide placeholder message        
    }
    toolsButton->show();

    QTextEdit::focusInEvent(event); // process standard event: allows context menu


    qDebug() << "Composer box got focus";
}



void Composer::keyPressEvent(QKeyEvent *event)
{
    //qDebug() << "Composer::keyPressEvent()";


    // Control+Enter = Send message (post)
    if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
      && event->modifiers() == Qt::ControlModifier)
    {
        qDebug() << "Control+Enter was pressed";
        emit editingFinished();
    }
    else if (event->key() == Qt::Key_Escape)
    {
        qDebug() << "Escape was pressed";
        if (this->toPlainText().isEmpty())
        {
            qDebug() << "There was no text, canceling post";
            this->cancelPost();
        }
    }
    else
    {
        QTextEdit::keyPressEvent(event);
    }
}


/*
 * For custom context menu
 *
 */
void Composer::contextMenuEvent(QContextMenuEvent *event)
{
    this->customContextMenu = this->createStandardContextMenu();

    // tools menu before default context menu
    customContextMenu->insertMenu(customContextMenu->actions().at(0),
                                  toolsMenu);
    customContextMenu->insertSeparator(customContextMenu->actions().at(1));

    // And options added after default context menu
    customContextMenu->addSeparator();


    customContextMenu->addAction(pastePlaintextAction);
    if (this->canPaste())
    {
        pastePlaintextAction->setEnabled(true);
    }
    else
    {
        pastePlaintextAction->setDisabled(true);
    }

    customContextMenu->exec(event->globalPos());

    // FIXME: Possible leak... should delete customContextMenu?
    customContextMenu->deleteLater();

    event->accept();
}


/*
 * Intervene when pasting, so we can turn links into real HTML links
 *
 */
void Composer::insertFromMimeData(const QMimeData *source)
{
    QString pastedText = source->text().trimmed();

    if (pastedText.startsWith("http://")
     || pastedText.startsWith("https://"))
    {
        this->insertLink(pastedText);
    }
    else // Just paste original contents
    {
        QTextEdit::insertFromMimeData(source);
    }
}



/*************************************************************************/
/****************************** SLOTS ************************************/
/*************************************************************************/



/*
 * Remove text formatting from selection, bold, italic, etc.
 *
 */
void Composer::makeNormal()
{
    QTextCharFormat charFormat;
    charFormat.clearForeground();
    charFormat.clearBackground();
    this->setCurrentCharFormat(charFormat);

    this->setFocus();
}



/*
 * Make selected text bold
 *
 */
void Composer::makeBold()
{
    //qDebug() << this->textCursor().selectionStart() << " -> "  << this->textCursor().selectionEnd();
    QTextCharFormat charFormat;
    if (this->currentCharFormat().fontWeight() == QFont::Bold)
    {
        charFormat.setFontWeight(QFont::Normal);
    }
    else
    {
        charFormat.setFontWeight(QFont::Bold);
    }
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus(); // give focus back to text editor
}


/*
 * Make selected text italic
 *
 */
void Composer::makeItalic()
{
    QTextCharFormat charFormat;
    charFormat.setFontItalic(!this->currentCharFormat().fontItalic());
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus();
}



/*
 * Underline selected text
 *
 */
void Composer::makeUnderline()
{
    QTextCharFormat charFormat;
    charFormat.setFontUnderline(!this->currentCharFormat().fontUnderline());
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus();
}

/*
 * Strike out selected text
 *
 */
void Composer::makeStrikethrough()
{
    QTextCharFormat charFormat;
    charFormat.setFontStrikeOut(!this->currentCharFormat().fontStrikeOut());
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus();
}

/*
 * Turn the selected text into an <h2> header
 *
 */
void Composer::makeHeader()
{
    QString selectedText = this->textCursor().selectedText();

    if (!selectedText.isEmpty())
    {
        this->textCursor().removeSelectedText();
        this->insertHtml("<h2>" + selectedText + "</h2> ");
    }

    this->setFocus();
}



/*
 * Put selected text into a <pre> block
 *
 */
void Composer::makePreformatted()
{
    QString selectedText = this->textCursor().selectedText();

    if (!selectedText.isEmpty())
    {
        this->textCursor().removeSelectedText();
        this->insertHtml("<pre>" + selectedText + "</pre> ");
    }

    this->setFocus();
}

/*
 * Mark selected as quoted, using <blockquote>
 *
 */
void Composer::makeQuote()
{
    QString selectedText = this->textCursor().selectedText();

    if (!selectedText.isEmpty())
    {
        this->textCursor().removeSelectedText();
        this->insertHtml("<br />"
                         "<blockquote>&ldquo;" + selectedText + "&rdquo;</blockquote> ");
    }
    // FIXME: Qt's HTML changes <blockquote> into its own formatting
    // so other clients and the web UI might not display it as other people's blockquote's

    this->setFocus();
}




/*
 * Convert selected text into a link
 *
 */
void Composer::makeLink()
{
    QString selectedText = this->textCursor().selectedText();
    QString link;


    if (selectedText.isEmpty())
    {
        link = QInputDialog::getText(this,
                                     tr("Insert a link"),
                                     tr("Type or paste a web address here.\n"),
                                     QLineEdit::Normal,
                                     "http://");
    }
    else
    {
        link = QInputDialog::getText(this,
                                     tr("Make a link from selected text"),
                                     tr("Type or paste a web address here.\n"
                                        "The selected text (%1) will be converted to a link.").arg(selectedText),
                                     QLineEdit::Normal,
                                     "http://");
    }


    if (!link.isEmpty())
    {
        link = link.trimmed(); // Remove possible spaces before or after

        this->textCursor().removeSelectedText();
        this->insertLink(link, selectedText);
    }
    else
    {
        qDebug() << "makeLink(): Invalid URL";
    }

    this->setFocus();
}



/*
 * Insert an image from a URL
 *
 */
void Composer::insertImage()
{
    // FIXME: should make sure there is no text selected

    QString imageURL;
    imageURL = QInputDialog::getText(this,
                                     tr("Insert an image from a URL"),
                                     tr("Type or paste the image address here.\n"),
                                     QLineEdit::Normal,
                                     "http://");


    if (!imageURL.isEmpty())
    {
        if (imageURL.startsWith("http://")
         || imageURL.startsWith("https://"))
        {
            this->insertHtml("<img src=\"" + imageURL + "\" />");
        }
        else
        {
            QMessageBox::warning(this, tr("Error: Invalid URL"),
                                 tr("The address you entered (%1) "
                                    "is not valid.\n"
                                    "Image addresses should begin "
                                    "with http:// or https://").arg(imageURL));
        }
    }
    else
    {
        qDebug() << "insertImage(): Image URL is empty";
    }

    this->setFocus();
}

/*
 * Insert a horizontal line, <hr>
 *
 */
void Composer::insertLine()
{
    this->insertHtml("<hr><br>");

    this->setFocus();
}



void Composer::insertSymbol(QAction *action)
{
    this->insertPlainText(action->text());
    this->insertPlainText(" ");

    this->setFocus();
}



void Composer::pasteAsPlaintext()
{
    QString subtype("plain");
    QString clipboardContents = QApplication::clipboard()->text(subtype,
                                                                QClipboard::Clipboard);

    this->makeNormal(); // Ensure normal text before insterting
    this->insertPlainText(clipboardContents);
}




/*
 * Cancel editing of the post, clear it, return to minimum mode
 *
 */
void Composer::cancelPost()
{
    int cancelConfirmed = 1;

    if (this->document()->isEmpty())
    {
        cancelConfirmed = 0; // Cancelling doesn't need confirmation if it's empty
    }
    else
    {
        cancelConfirmed = QMessageBox::question(this,
                                                tr("Cancel message?"),
                                                tr("Are you sure you want to cancel this message?"),
                                                tr("&Yes, cancel it"), tr("&No"),
                                                "", 1, 1);

    }


    if (cancelConfirmed == 0)
    {
        this->erase();

        // emit signal to make Publisher go back to minimum mode
        emit editingCancelled();

        qDebug() << "Post canceled";
    }


}

==> dianara_1.1.orig.tar.gz: dianara-v1.1/src/composer.cpp <==
/*
 *   This file is part of Dianara
 *   Copyright 2012-2014  JanKusanagi <janjabber@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .
 */

#include "composer.h"


Composer::Composer(bool forPublisher)
{
    this->forPublisher = forPublisher;

    this->setAcceptRichText(true);
    this->setTabChangesFocus(true);

    // FIXME: do something about the links being styled with user's color
    //this->setStyleSheet("{ a:link { color: #FF0000;} }");

    QFont startConversationFont;
    startConversationFont.setPointSize(startConversationFont.pointSize() - 2);

    startConversationLabel = new QLabel(tr("Click here or press Control+N to post a note..."));
    startConversationLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    startConversationLabel->setFont(startConversationFont);

    // A menu to insert some Unicode symbols
    symbolsMenu = new QMenu(tr("Symbols"));
    symbolsMenu->setIcon(QIcon::fromTheme("character-set"));
    symbolsMenu->addAction(QString::fromUtf8("\342\230\272")); // Smiling face
    symbolsMenu->addAction(QString::fromUtf8("\342\230\271")); // Sad face
    symbolsMenu->addAction(QString::fromUtf8("\342\231\245")); // Heart
    symbolsMenu->addAction(QString::fromUtf8("\342\231\253")); // Musical note
    symbolsMenu->addAction(QString::fromUtf8("\342\230\225")); // Coffee
    symbolsMenu->addAction(QString::fromUtf8("\342\234\224")); // Check mark
    symbolsMenu->addAction(QString::fromUtf8("\342\230\205")); // Black star
    symbolsMenu->addAction(QString::fromUtf8("\342\254\205")); // Arrow to the left
    symbolsMenu->addAction(QString::fromUtf8("\342\236\241")); // Arrow to the right
    connect(symbolsMenu, SIGNAL(triggered(QAction*)),
            this, SLOT(insertSymbol(QAction*)));


    toolsMenu = new QMenu(tr("Formatting"));
    toolsMenu->addAction(QIcon::fromTheme(""),
                         tr("Normal"),
                         this,
                         SLOT(makeNormal()));
    toolsMenu->addAction(QIcon::fromTheme("format-text-bold"),
                         tr("Bold"),
                         this,
                         SLOT(makeBold()),
                         QKeySequence("Ctrl+B"));
    toolsMenu->addAction(QIcon::fromTheme("format-text-italic"),
                         tr("Italic"),
                         this,
                         SLOT(makeItalic()),
                         QKeySequence("Ctrl+I"));
    toolsMenu->addAction(QIcon::fromTheme("format-text-underline"),
                         tr("Underline"),
                         this,
                         SLOT(makeUnderline()),
                         QKeySequence("Ctrl+U"));
    toolsMenu->addAction(QIcon::fromTheme("format-text-strikethrough"),
                         tr("Strikethrough"),
                         this,
                         SLOT(makeStrikethrough()));

    toolsMenu->addSeparator();

    toolsMenu->addAction(QIcon::fromTheme("format-font-size-more"),
                         tr("Header"),
                         this,
                         SLOT(makeHeader()),
                         QKeySequence("Ctrl+H"));
    toolsMenu->addAction(QIcon::fromTheme("format-justify-fill"),
                         tr("Preformatted block"),
                         this,
                         SLOT(makePreformatted()));
    toolsMenu->addAction(QIcon::fromTheme("format-text-italic"),
                         tr("Quote block"),
                         this,
                         SLOT(makeQuote()));

    toolsMenu->addSeparator();

    toolsMenu->addAction(QIcon::fromTheme("insert-link"),
                         tr("Make a link"),
                         this,
                         SLOT(makeLink()),
                         QKeySequence("Ctrl+L"));
    toolsMenu->addAction(QIcon::fromTheme("insert-image"),
                         tr("Insert an image from a web site"),
                         this,
                         SLOT(insertImage()),
                         QKeySequence("Ctrl+P"));
    toolsMenu->addAction(QIcon::fromTheme("insert-horizontal-rule"),
                         tr("Insert line"),
                         this,
                         SLOT(insertLine()));

    toolsMenu->addSeparator();

    toolsMenu->addMenu(symbolsMenu);


    toolsButton = new QPushButton(QIcon::fromTheme("format-list-ordered"),
                                  tr("&Format",
                                     "Button for text formatting and related options"));
    toolsButton->setMenu(toolsMenu);
    toolsButton->setToolTip(tr("Text Formatting Options"));
    toolsButton->setHidden(true); // Hidden on startup (Publisher in miniMode)


    // Extra action for the context menu, paste as plaintext
    this->pastePlaintextAction = new QAction(QIcon::fromTheme("paste"),
                                             tr("Paste Text Without Formatting"),
                                             this);

    pastePlaintextAction->setShortcut(QKeySequence("Ctrl+Shift+V"));
    connect(pastePlaintextAction, SIGNAL(triggered()),
            this, SLOT(pasteAsPlaintext()));

    // Add action to this object, in order for the shortcut to work
    // Otherwise, the action isn't available until the context menu is present
    this->addAction(pastePlaintextAction);

    mainLayout = new QHBoxLayout();

    if (this->forPublisher)  // Publisher mode
    {
        this->setToolTip(tr("Type a message here to post it"));

        mainLayout->addWidget(startConversationLabel, 1, Qt::AlignLeft | Qt::AlignVCenter);
    }
    else                     // or Commenter mode
    {
        this->setToolTip(tr("Type a comment here"));
    }
    this->setLayout(mainLayout);

    qDebug() << "Composer box created";
}


Composer::~Composer()
{
    qDebug() << "Composer box destroyed";
}



void Composer::erase()
{
    this->clear();

    if (this->forPublisher)
    {
        startConversationLabel->show();

        toolsButton->hide();  /// FIXME: maybe move this line to focusOutEvent
    }
}


void Composer::insertLink(QString url, QString title)
{
    if (title.isEmpty())
    {
        title = url;
    }

    this->insertHtml("<a href=\"" + url + "\">"
                     "<span style=\"color:#0088CC\">"
                     + title + "</span></a> "); // Important, space after!
}

int Composer::getMessageLabelHeight()
{
    // kinda TMP / FIXME
    return (this->startConversationLabel->font().pointSize() * 4) + 4;
}


/*
 * Send a signal when getting focus
 *
 */
void Composer::focusInEvent(QFocusEvent *event)
{
    emit focusReceived(); // inform Publisher() or Commenter() that we have focus

    if (this->forPublisher)
    {
        startConversationLabel->hide(); // hide placeholder message        
    }
    toolsButton->show();

    QTextEdit::focusInEvent(event); // process standard event: allows context menu


    qDebug() << "Composer box got focus";
}



void Composer::keyPressEvent(QKeyEvent *event)
{
    //qDebug() << "Composer::keyPressEvent()";


    // Control+Enter = Send message (post)
    if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
      && event->modifiers() == Qt::ControlModifier)
    {
        qDebug() << "Control+Enter was pressed";
        emit editingFinished();
    }
    else if (event->key() == Qt::Key_Escape)
    {
        qDebug() << "Escape was pressed";
        if (this->toPlainText().isEmpty())
        {
            qDebug() << "There was no text, canceling post";
            this->cancelPost();
        }
    }
    else
    {
        QTextEdit::keyPressEvent(event);
    }
}


/*
 * For custom context menu
 *
 */
void Composer::contextMenuEvent(QContextMenuEvent *event)
{
    this->customContextMenu = this->createStandardContextMenu();

    // tools menu before default context menu
    customContextMenu->insertMenu(customContextMenu->actions().at(0),
                                  toolsMenu);
    customContextMenu->insertSeparator(customContextMenu->actions().at(1));

    // And options added after default context menu
    customContextMenu->addSeparator();


    customContextMenu->addAction(pastePlaintextAction);
    if (this->canPaste())
    {
        pastePlaintextAction->setEnabled(true);
    }
    else
    {
        pastePlaintextAction->setDisabled(true);
    }

    customContextMenu->exec(event->globalPos());

    // FIXME: Possible leak... should delete customContextMenu?
    customContextMenu->deleteLater();

    event->accept();
}


/*
 * Intervene when pasting, so we can turn links into real HTML links
 *
 */
void Composer::insertFromMimeData(const QMimeData *source)
{
    QString pastedText = source->text().trimmed();

    if (pastedText.startsWith("http://")
     || pastedText.startsWith("https://"))
    {
        this->insertLink(pastedText);
    }
    else // Just paste original contents
    {
        QTextEdit::insertFromMimeData(source);
    }
}



/*************************************************************************/
/****************************** SLOTS ************************************/
/*************************************************************************/



/*
 * Remove text formatting from selection, bold, italic, etc.
 *
 */
void Composer::makeNormal()
{
    QTextCharFormat charFormat;
    charFormat.clearForeground();
    charFormat.clearBackground();
    this->setCurrentCharFormat(charFormat);

    this->setFocus();
}



/*
 * Make selected text bold
 *
 */
void Composer::makeBold()
{
    //qDebug() << this->textCursor().selectionStart() << " -> "  << this->textCursor().selectionEnd();
    QTextCharFormat charFormat;
    if (this->currentCharFormat().fontWeight() == QFont::Bold)
    {
        charFormat.setFontWeight(QFont::Normal);
    }
    else
    {
        charFormat.setFontWeight(QFont::Bold);
    }
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus(); // give focus back to text editor
}


/*
 * Make selected text italic
 *
 */
void Composer::makeItalic()
{
    QTextCharFormat charFormat;
    charFormat.setFontItalic(!this->currentCharFormat().fontItalic());
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus();
}



/*
 * Underline selected text
 *
 */
void Composer::makeUnderline()
{
    QTextCharFormat charFormat;
    charFormat.setFontUnderline(!this->currentCharFormat().fontUnderline());
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus();
}

/*
 * Strike out selected text
 *
 */
void Composer::makeStrikethrough()
{
    QTextCharFormat charFormat;
    charFormat.setFontStrikeOut(!this->currentCharFormat().fontStrikeOut());
    this->mergeCurrentCharFormat(charFormat);

    this->setFocus();
}

/*
 * Turn the selected text into an <h2> header
 *
 */
void Composer::makeHeader()
{
    QString selectedText = this->textCursor().selectedText();

    if (!selectedText.isEmpty())
    {
        this->textCursor().removeSelectedText();
        this->insertHtml("<h2>" + selectedText + "</h2> ");
    }

    this->setFocus();
}



/*
 * Put selected text into a <pre> block
 *
 */
void Composer::makePreformatted()
{
    QString selectedText = this->textCursor().selectedText();

    if (!selectedText.isEmpty())
    {
        this->textCursor().removeSelectedText();
        this->insertHtml("<pre>" + selectedText + "</pre> ");
    }

    this->setFocus();
}

/*
 * Mark selected as quoted, using <blockquote>
 *
 */
void Composer::makeQuote()
{
    QString selectedText = this->textCursor().selectedText();

    if (!selectedText.isEmpty())
    {
        this->textCursor().removeSelectedText();
        this->insertHtml("<br />"
                         "<blockquote>&ldquo;" + selectedText + "&rdquo;</blockquote> ");
    }
    // FIXME: Qt's HTML changes <blockquote> into its own formatting
    // so other clients and the web UI might not display it as other people's blockquote's

    this->setFocus();
}




/*
 * Convert selected text into a link
 *
 */
void Composer::makeLink()
{
    QString selectedText = this->textCursor().selectedText();
    QString link;


    if (selectedText.isEmpty())
    {
        link = QInputDialog::getText(this,
                                     tr("Insert a link"),
                                     tr("Type or paste a web address here.\n"),
                                     QLineEdit::Normal,
                                     "http://");
    }
    else
    {
        link = QInputDialog::getText(this,
                                     tr("Make a link from selected text"),
                                     tr("Type or paste a web address here.\n"
                                        "The selected text (%1) will be converted "
                                        "to a link.").arg(selectedText),
                                     QLineEdit::Normal,
                                     "http://");
    }


    if (!link.isEmpty())
    {
        link = link.trimmed(); // Remove possible spaces before or after

        this->textCursor().removeSelectedText();
        this->insertLink(link, selectedText);
    }
    else
    {
        qDebug() << "makeLink(): Invalid URL";
    }

    this->setFocus();
}



/*
 * Insert an image from a URL
 *
 */
void Composer::insertImage()
{
    // FIXME: should make sure there is no text selected

    QString imageURL;
    imageURL = QInputDialog::getText(this,
                                     tr("Insert an image from a URL"),
                                     tr("Type or paste the image address here.\n"),
                                     QLineEdit::Normal,
                                     "http://");


    if (!imageURL.isEmpty())
    {
        if (imageURL.startsWith("http://")
         || imageURL.startsWith("https://"))
        {
            this->insertHtml("<img src=\"" + imageURL + "\" />");
        }
        else
        {
            QMessageBox::warning(this, tr("Error: Invalid URL"),
                                 tr("The address you entered (%1) "
                                    "is not valid.\n"
                                    "Image addresses should begin "
                                    "with http:// or https://").arg(imageURL));
        }
    }
    else
    {
        qDebug() << "insertImage(): Image URL is empty";
    }

    this->setFocus();
}

/*
 * Insert a horizontal line, <hr>
 *
 */
void Composer::insertLine()
{
    this->insertHtml("<hr><br>");

    this->setFocus();
}



void Composer::insertSymbol(QAction *action)
{
    this->insertPlainText(action->text());
    this->insertPlainText(" ");

    this->setFocus();
}



void Composer::pasteAsPlaintext()
{
    QString subtype("plain");
    QString clipboardContents = QApplication::clipboard()->text(subtype,
                                                                QClipboard::Clipboard);

    this->makeNormal(); // Ensure normal text before insterting
    this->insertPlainText(clipboardContents);
}




/*
 * Cancel editing of the post, clear it, return to minimum mode
 *
 */
void Composer::cancelPost()
{
    int cancelConfirmed = 1;

    if (this->document()->isEmpty())
    {
        cancelConfirmed = 0; // Cancelling doesn't need confirmation if it's empty
    }
    else
    {
        cancelConfirmed = QMessageBox::question(this,
                                                tr("Cancel message?"),
                                                tr("Are you sure you want to cancel this message?"),
                                                tr("&Yes, cancel it"), tr("&No"),
                                                "", 1, 1);

    }


    if (cancelConfirmed == 0)
    {
        this->erase();

        // emit signal to make Publisher go back to minimum mode
        emit editingCancelled();

        qDebug() << "Post canceled";
    }


}

QPushButton *Composer::getToolsButton()
{
    return this->toolsButton;
}
